@@ -1,9 +1,8 @@
-#' Under-sampling by removing points near other classes.
+#' Remove Points Near Other Classes
 #'
-#' `step_nearmiss` creates a *specification* of a recipe
-#'  step that removes majority class instances by undersampling points
-#'  in the majority class based on their distance to other points in the
-#'  same class.
+#' `step_nearmiss()` creates a *specification* of a recipe step that removes
+#' majority class instances by undersampling points in the majority class based
+#' on their distance to other points in the same class.
 #'
 #' @inheritParams recipes::step_center
 #' @inheritParams step_downsample
@@ -25,8 +24,8 @@
 #'  the variable used to sample.
 #'
 #' @details
-#' This methods retained the points form the majority classes which has the
-#' smallest mean distance to the k nearest points in the other classes.
+#' This method retains the points from the majority class which have the
+#' smallest mean distance to the k nearest points in the minority class.
 #'
 #' All columns in the data are sampled and returned by [juice()]
 #'  and [bake()].
@@ -37,31 +36,60 @@
 #'  option `skip = TRUE` so that the extra sampling is _not_
 #'  conducted outside of the training set.
 #'
+#' # Tidying
+#'
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble with columns `terms`
+#' (the selectors or variables selected) will be returned.
+#'
+#' ```{r, echo = FALSE, results="asis"}
+#' step <- "step_nearmiss"
+#' result <- knitr::knit_child("man/rmd/tunable-args.Rmd")
+#' cat(result)
+#' ```
+#'
+#' @template case-weights-not-supported
+#'
 #' @references Inderjeet Mani and I Zhang. knn approach to unbalanced data
 #' distributions: a case study involving information extraction. In Proceedings
 #' of workshop on learning from imbalanced datasets, 2003.
 #'
-#' @keywords datagen
-#' @concept preprocessing
-#' @concept subsampling
+#' @seealso [nearmiss()] for direct implementation
+#' @family Steps for under-sampling
+#'
 #' @export
 #' @examples
 #' library(recipes)
 #' library(modeldata)
-#' data(okc)
+#' data(hpc_data)
 #'
-#' sort(table(okc$Class, useNA = "always"))
+#' hpc_data0 <- hpc_data %>%
+#'   select(-protocol, -day)
 #'
-#' ds_rec <- recipe(Class ~ age + height, data = okc) %>%
-#'   step_meanimpute(all_predictors()) %>%
-#'   step_nearmiss(Class) %>%
+#' orig <- count(hpc_data0, class, name = "orig")
+#' orig
+#'
+#' up_rec <- recipe(class ~ ., data = hpc_data0) %>%
+#'   # Bring the majority levels down to about 1000 each
+#'   # 1000/259 is approx 3.862
+#'   step_nearmiss(class, under_ratio = 3.862) %>%
 #'   prep()
 #'
-#' sort(table(bake(ds_rec, new_data = NULL)$Class, useNA = "always"))
+#' training <- up_rec %>%
+#'   bake(new_data = NULL) %>%
+#'   count(class, name = "training")
+#' training
 #'
-#' # since `skip` defaults to TRUE, baking the step has no effect
-#' baked_okc <- bake(ds_rec, new_data = okc)
-#' table(baked_okc$Class, useNA = "always")
+#' # Since `skip` defaults to TRUE, baking the step has no effect
+#' baked <- up_rec %>%
+#'   bake(new_data = hpc_data0) %>%
+#'   count(class, name = "baked")
+#' baked
+#'
+#' # Note that if the original data contained more rows than the
+#' # target n (= ratio * majority_n), the data are left alone:
+#' orig %>%
+#'   left_join(training, by = "class") %>%
+#'   left_join(baked, by = "class")
 #'
 #' library(ggplot2)
 #'
@@ -71,7 +99,7 @@
 #'   xlim(c(1, 15)) +
 #'   ylim(c(1, 15))
 #'
-#' recipe(class ~ ., data = circle_example) %>%
+#' recipe(class ~ x + y, data = circle_example) %>%
 #'   step_nearmiss(class) %>%
 #'   prep() %>%
 #'   bake(new_data = NULL) %>%
@@ -88,12 +116,13 @@ step_nearmiss <-
     add_step(
       recipe,
       step_nearmiss_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         column = column,
         under_ratio = under_ratio,
         neighbors = neighbors,
+        predictors = NULL,
         skip = skip,
         seed = seed,
         id = id
@@ -102,8 +131,8 @@ step_nearmiss <-
   }
 
 step_nearmiss_new <-
-  function(terms, role, trained, column, under_ratio, neighbors, skip, seed,
-           id) {
+  function(terms, role, trained, column, under_ratio, neighbors, predictors,
+           skip, seed, id) {
     step(
       subclass = "nearmiss",
       terms = terms,
@@ -112,6 +141,7 @@ step_nearmiss_new <-
       column = column,
       under_ratio = under_ratio,
       neighbors = neighbors,
+      predictors = predictors,
       skip = skip,
       id = id,
       seed = seed,
@@ -121,20 +151,21 @@ step_nearmiss_new <-
 
 #' @export
 prep.step_nearmiss <- function(x, training, info = NULL, ...) {
-  col_name <- terms_select(x$terms, info = info)
-  if (length(col_name) != 1) {
-    rlang::abort("Please select a single factor variable.")
-  }
-  if (!is.factor(training[[col_name]])) {
-    rlang::abort(paste0(col_name, " should be a factor variable."))
+  col_name <- recipes_eval_select(x$terms, training, info)
+
+  if (length(col_name) > 1) {
+    rlang::abort("The selector should select at most a single variable")
   }
 
-  check_type(select(training, -col_name), TRUE)
-
-  if (any(map_lgl(training, ~ any(is.na(.x))))) {
-    rlang::abort("`NA` values are not allowed when using `step_nearmiss`")
+  if (length(col_name) == 1) {
+    check_column_factor(training, col_name)
   }
 
+  predictors <- setdiff(get_from_info(info, "predictor"), col_name)
+
+  check_type(training[, predictors], types = c("double", "integer"))
+  check_na(select(training, all_of(c(col_name, predictors))))
+
   step_nearmiss_new(
     terms = x$terms,
     role = x$role,
@@ -142,6 +173,7 @@ prep.step_nearmiss <- function(x, training, info = NULL, ...) {
     column = col_name,
     under_ratio = x$under_ratio,
     neighbors = x$neighbors,
+    predictors = predictors,
     skip = x$skip,
     seed = x$seed,
     id = x$id
@@ -150,42 +182,53 @@ prep.step_nearmiss <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_nearmiss <- function(object, new_data, ...) {
+  col_names <- unique(c(object$predictors, object$column))
+  check_new_data(col_names, object, new_data)
+
+  if (length(object$column) == 0L) {
+    # Empty selection
+    return(new_data)
+  }
+
+  ignore_vars <- setdiff(names(new_data), col_names)
 
   # nearmiss with seed for reproducibility
   with_seed(
     seed = object$seed,
     code = {
       original_levels <- levels(new_data[[object$column]])
-      new_data <- nearmiss(
+      new_data <- nearmiss_impl(
         df = new_data,
         var = object$column,
+        ignore_vars = ignore_vars,
         k = object$neighbors,
         under_ratio = object$under_ratio
       )
-      new_data[[object$column]] <- factor(new_data[[object$column]],
-                                          levels = original_levels)
+      new_data[[object$column]] <- factor(
+        new_data[[object$column]],
+        levels = original_levels
+      )
     }
   )
 
-  as_tibble(new_data)
+  new_data
 }
 
 #' @export
 print.step_nearmiss <-
   function(x, width = max(20, options()$width - 26), ...) {
-    cat("NEARMISS-1 based on ", sep = "")
-    printer(x$column, x$terms, x$trained, width = width)
+    title <- "NEARMISS-1 based on "
+    print_step(x$column, x$terms, x$trained, title, width)
     invisible(x)
   }
 
-#' @rdname step_nearmiss
+#' @rdname tidy.recipe
 #' @param x A `step_nearmiss` object.
 #' @export
 tidy.step_nearmiss <- function(x, ...) {
   if (is_trained(x)) {
-    res <- tibble(terms = x$column)
-  }
-  else {
+    res <- tibble(terms = unname(x$column))
+  } else {
     term_names <- sel2char(x$terms)
     res <- tibble(terms = unname(term_names))
   }
@@ -193,7 +236,20 @@ tidy.step_nearmiss <- function(x, ...) {
   res
 }
 
-
+#' @export
+#' @rdname tunable_themis
+tunable.step_nearmiss <- function(x, ...) {
+  tibble::tibble(
+    name = c("under_ratio", "neighbors"),
+    call_info = list(
+      list(pkg = "dials", fun = "under_ratio"),
+      list(pkg = "dials", fun = "neighbors", range = c(1, 10))
+    ),
+    source = "recipe",
+    component = "step_nearmiss",
+    component_id = x$id
+  )
+}
 
 #' @rdname required_pkgs.step
 #' @export
